/*============================================================================
  File:     4b_Find_Plan.sql

  SQL Server Versions: 2012 onwards
------------------------------------------------------------------------------
  Written by Erin Stellato, SQLskills.com
  
  (c) 2013, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

IF EXISTS (SELECT 1 FROM sys.server_event_sessions WHERE name = 'Research_SP')
	DROP EVENT SESSION [Research_SP] ON SERVER;
GO

CREATE EVENT SESSION [Research_SP] 
	ON SERVER 
	ADD EVENT sqlserver.rpc_completed(
		ACTION(sqlserver.client_app_name,sqlserver.query_hash,sqlserver.query_plan_hash)
		WHERE ([object_name]=N'usp_GetProductInfo')),
	ADD EVENT sqlserver.sp_statement_completed(
		ACTION(sqlserver.client_app_name,sqlserver.query_hash,sqlserver.query_plan_hash)
		WHERE ([object_id]=(791673868))) 
	ADD TARGET package0.event_file(SET filename=N'C:\Temp\Research_SP.xel',max_file_size=(1024),max_rollover_files=(1))
WITH (MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,MAX_DISPATCH_LATENCY=30 SECONDS,MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,TRACK_CAUSALITY=ON,STARTUP_STATE=OFF)
GO



/*
	start session
*/
ALTER EVENT SESSION [Research_SP]
	ON SERVER
	STATE=START;
	GO


/*
	run scripts
	watch data
	get query plan hash
*/


/* stop and drop*/
ALTER EVENT SESSION [Research_SP]
	ON SERVER
	STATE=STOP;
	GO
	
DROP EVENT SESSION [Research_SP]
	ON SERVER;
	GO



SELECT *
FROM sys.dm_exec_query_stats
WHERE query_plan_hash = CAST(<query_plan_hash_here> AS BIGINT);


SELECT
	OBJECT_NAME([qp].[objectid],[qp].[dbid]),
    [qs].[sql_handle],
	[qs].[plan_handle],
	[qs].[query_hash],
	[qs].[query_plan_hash],
	[qs].[creation_time],
	[qs].[execution_count],
	(DATEDIFF(ms,[qs].[creation_time],GETDATE())/[qs].[execution_count]) [Executions/ms],
	[qs].[last_execution_time],
	[qs].[last_logical_reads],
	[qs].[last_worker_time],
	[qs].[min_logical_reads],
	[qs].[max_logical_reads],
	[qs].[min_worker_time],
	[qs].[max_worker_time],
	[qs].[min_rows],
	[qs].[max_rows],
	[qs].[total_logical_reads],
	[qs].[total_worker_time],
	[qp].[query_plan]
FROM [sys].[dm_exec_query_stats] [qs]
CROSS APPLY [sys].[dm_exec_query_plan]([qs].[plan_handle]) [qp]
WHERE [query_plan_hash] = CAST(<query_plan_hash_here> AS BIGINT);


USE [AdventureWorks2012];
GO

--Enable Actual Execution Plan
SET STATISTICS IO ON;

--Get ProductID for query with low duration
EXEC [Sales].[usp_GetProductInfo] <enterProductID>
--


--Get ProductID for query with high duration
EXEC [Sales].[usp_GetProductInfo] <enterProductID>
--


EXEC [Sales].[usp_GetProductInfo] <enterProductID> WITH RECOMPILE;


DBCC FREEPROCCACHE;


DBCC FREEPROCCACHE (<Enter_Really_Crazy_Long_Plan_Handle_That_You_Can_Never_Memorize_Here>)


SELECT ProductID, COUNT(*)
FROM Sales.SalesOrderDetail
GROUP BY ProductID
ORDER BY COUNT(*) DESC